

/*  Copyright (C) 2002 Secure Systems Lab, Stony Brook. 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (See file COPYING in the main directory); 
    if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*   Written by Tapan Kamat tkamat@cs.sunysb.edu */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;

/**
 * A subclass of DefaultTreeCellRenderer that handles the display of checkboxes
 * for nodes in the tree.
 * All that is required to have a node display a checkbox is for the user object
 * that that is used to create the DefaultMutableTreeNode to implement the
 * Checkable interface. When the user selects or deselects the node in the
 * tree the object's setChecked(boolean v) method will be called.
 *
 * 
 */
public class CheckableTreeCellRenderer extends DefaultTreeCellRenderer {

  /**
   * The static constant for check marks.
   */
  public static final int CHECKMARK = 0;

  /**
   * The static constant for x marks.
   */
  public static final int XMARK = 1;

  /**
   * Returns a static instance of the CheckableTreeCellRenderer.
   */
  private static CheckableTreeCellRenderer instance;
  public static CheckableTreeCellRenderer getStaticInstance() {
    if(instance == null) {
      instance = new CheckableTreeCellRenderer();
    }
    return instance;
  }

  /**
   * Create a CheckableTreeCellRenderer. The default check style is to use
   * checkmarks rather than x marks.
   */
  public CheckableTreeCellRenderer() {
    checkStyle = CHECKMARK;
  }

  /**
   * Return the component that will be used to render the tree cell.
   *
   * @param tree The tree that will use the renderer
   * @param The node in the tree for which a renderer is needed.
   * @param sel Whether the node should be drawn selected.
   * @param expanded Whether the node should be drawn expanded.
   * @param leaf Whether the node being rendered is a leaf node.
   * @param row The row number for the node being rendered.
   * @param hasFocus Whether the node being rendered has focus
   * @return The component to use in the tree.
   */
  public Component getTreeCellRendererComponent(JTree tree, Object value,
	                                              boolean sel, boolean expanded,
                                                boolean leaf, int row, boolean hasFocus) {
    checkVisible = false;
    if(value instanceof DefaultMutableTreeNode) {
      DefaultMutableTreeNode dMTN = (DefaultMutableTreeNode)value;
      Object o = dMTN.getUserObject();
      if(o instanceof Checkable) {
        checkVisible = true;
        Checkable checkable = (Checkable)o;
        selected = checkable.isChecked();
      }
    }
    return super.getTreeCellRendererComponent(tree, value, sel, expanded,
                                              leaf, row, hasFocus);
  }

  /**
   * Sets the size of the checkboxes in the tree.
   *
   * @param boxSize The new size to use for the box width and height
   */
  public void setBoxSize(int boxSize) {
    this.boxSize = boxSize;
  }
  public int getBoxSize() {
    return boxSize;
  }

  /**
   * Sets the style of the checkboxes in the tree.
   *
   * @param checkStyle The new style for the checkboxes.
   * @throws IllegalArgumentException if the style is not one of CHECKMARK or
   *         XMARK.
   */
  public void setStyle(int checkStyle) {
    if (checkStyle != CHECKMARK && checkStyle != XMARK)
      throw new IllegalArgumentException();
    this.checkStyle = checkStyle;
  }
  public int getStyle() {
    return checkStyle;
  }

  /**
   * Determines whether the given point is inside the checkbox portion of the
   * renderer.
   *
   * @param p The point to check.
   * @returns True or false.
   */
  public boolean isPointInBox(Point p) {
    Rectangle rect = getBoxRectangle();
    int x1 = rect.x;
    int x2 = rect.x + rect.width;

    int why = Math.abs(rect.y);

    int y1 = rect.y;
    if(y1 < 0) {
      y1 = 0;
    }
    int y2 = y1 + rect.height;

    boolean retVal = true;
    if(p.x < x1) {
      retVal = false;
    }
    if(p.x > x2) {
      retVal = false;
    }

    if(p.y < y1) {
      retVal = false;
    }
    if(p.y > y2) {
      retVal = false;
    }

    return retVal;
  }

  /**
   * Get the preferred size of this component.
   */
  public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    if (checkVisible)
      dim.width += boxSize + HGAP;
    dim.width += 2;
    dim.height += 2;
    return dim;
  }

  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  /**
   * Figure out where the checkbox is.
   */
  protected Rectangle getBoxRectangle() {
    Rectangle boxRect = new Rectangle(0, 0, boxSize, boxSize);
    if (alignment == LEFT) {
      boxRect.x += 1;
      boxRect.y = (getHeight() - boxSize) / 2;
    }
    else if (alignment == CENTER) {
      Dimension dim = getPreferredSize();
      boxRect.x = (getWidth() - dim.width) / 2;
      boxRect.y = (getHeight() - boxSize) / 2;
    }
    return boxRect;
  }

  protected void paintComponent(Graphics g) {
    // If the checkmark is visible, we'll draw it first.
    if (checkVisible) {
      Rectangle boxRect = getBoxRectangle();

      drawBox(g, boxRect.x, boxRect.y, boxRect.width, boxRect.height);

      // If the check is selected, then draw the check mark.
      if (selected) {
        drawCheck(g, boxRect.x + 1, boxRect.y + 1, boxRect.width - 2, boxRect.height - 2);
      }

      super.paintBorder(g);

      // Now we want to shift the graphics x-axis to the right, so the superclass
      // can paint correctly.
      g.translate(boxSize + HGAP, 0);
    }
    else {
      super.paintBorder(g);
    }
    super.paintComponent(g);
  }

  protected void paintBorder(Graphics g) {
    // Do nothing...
  }

  protected void drawBox(Graphics g, int x, int y, int w, int h) {
    Color oldColor = g.getColor();
    g.setColor(Color.white);
    g.drawRect(x, y, w - 1, h - 1);
    g.setColor(Color.gray);
    g.drawLine(x, y, x+w-1, y);
    g.drawLine(x, y, x, y + h - 1);
    x++;
    y++;
    w--;
    h--;
    g.setColor(Color.lightGray);
    g.drawRect(x, y, w - 1, h - 1);
    //g.fillRect(x, y, w - 1, h - 1);
    g.setColor(Color.darkGray);
    g.drawLine(x, y, x+w-1, y);
    g.drawLine(x, y, x, y + h - 1);

    g.setColor(oldColor);
  }

  protected void drawCheck(Graphics g, int x, int y, int w, int h) {
    x++;
    y++;
    w--;
    h--;
    Color oldColor = g.getColor();
    x++; y++;
    g.setColor(Color.black);
    if (checkStyle == CHECKMARK) {
      int thickness = h / 3;
      int xbase = x + w / 3 - 1;
      for (int t = 1; t <= thickness; t++) {
        // short left line
        g.drawLine(xbase, y + h - t - 2, x, y + thickness - t + h / 2 - 2);
        // long right line
        g.drawLine(xbase, y + h - t - 2, x + w - 3, y + thickness - t);
      }
    }
    else if (checkStyle == XMARK) {
      // top-left to bottom-right
      g.drawLine(x, y, x + w - 3, y + h - 3);
      g.drawLine(x + 1, y, x + w - 3, y + h - 4);
      g.drawLine(x, y + 1, x + w - 4, y + h - 3);
      // bottom-left to top-right
      g.drawLine(x, y + h - 3, x + w - 3, y);
      g.drawLine(x + 1, y + h - 3, x + w - 3, y + 1);
      g.drawLine(x, y + h - 4, x + w - 4, y);
    }
    g.setColor(oldColor);
  }

  //----------------------------------------------------------------------------
  // Protected fields
  //----------------------------------------------------------------------------

  // The default box size
  protected int boxSize = 12;
  // The default checkstyle
  protected int checkStyle = CHECKMARK;

  // The gap between the box and the label area
  protected static final int HGAP = 4;

  // The default alignment
  protected int alignment = LEFT;

  protected boolean checkVisible = true;
  protected boolean selected = true;

}
